
float4x4 mWorldViewProj;  // World * View * Projection transformation
float4x4 mInvWorld;       // Inverted world matrix
float4x4 mTransWorld;     // Transposed world matrix
float3 mLightPos;         // Light position
float4 mLightColor;       // Light color


// Vertex shader output structure
struct VS_OUTPUT
{
	float4 Position   : POSITION;   // vertex position 
	float4 Diffuse    : COLOR0;     // vertex diffuse color
	float4 Ambient    : COLOR1;     // vertex diffuse color
};


VS_OUTPUT vertexMain( in float4 vColor    : COLOR0,
                      in float4 vPosition : POSITION,
                      in float3 vNormal   : NORMAL )
{
	VS_OUTPUT Output;

	// transform position to clip space 
	Output.Position = mul(vPosition, mWorldViewProj);
	
	// transform normal 
	float3 normal = vNormal;
	normal.z = -normal.z;
	normal = mul(normal, mInvWorld);
	normal.z = -normal.z;
	
	// renormalize normal 
	normal = normalize(normal);
	
	// position in world coodinates
	float3 worldpos = mul(mTransWorld, vPosition);
	
	// calculate light vector, vtxpos - lightpos
	float3 lightVector = float3(0.0, 0.0, 0.0) - mLightPos;
	
	// normalize light vector 
	lightVector = normalize(lightVector);
	
	// calculate light color 
	float3 tmp = dot(-lightVector, normal);
	tmp = lit(tmp.x, tmp.y, 1.0);
	
	tmp = mLightColor * tmp.y;
	Output.Diffuse = float4(tmp.x, tmp.y, tmp.z, 0);
	Output.Ambient = vColor;
	
	return Output;
}



// Pixel shader output structure
struct PS_OUTPUT
{
    float4 RGBColor : COLOR0;  // Pixel color    
};


PS_OUTPUT pixelMain( float4 Position : POSITION,
                     float4 Diffuse  : COLOR0,
                     float4 Ambient  : COLOR1 ) 
{ 
	PS_OUTPUT Output;

	float4 col = Ambient;  // sample color map
	float multip = max(Diffuse.x, Diffuse.y);

	multip = max(multip, Diffuse.z);

	multip = 1.0 - multip;
	multip = multip / 2.0;
	
	// multiply with diffuse and do other senseless operations
	Output.RGBColor = (Diffuse + float4(multip, multip, multip, 1.0)) * col;
	Output.RGBColor *= 2.0;

	return Output;
}

