
float4x4 mWorldViewProj;  // World * View * Projection transformation
float4x4 mInvWorld;       // Inverted world matrix
float4x4 mTransWorld;     // Transposed world matrix
float3 mLightPos;         // Light position
float3 eyePosition;
float4 mLightColor;       // Light color


// Vertex shader output structure
struct VS_OUTPUT
{
	float4 Position    : POSITION;   // vertex position 
	float4 Diffuse     : COLOR0;     // vertex diffuse color
	float2 TexCoord0   : TEXCOORD0;  // tex coords
	float2 TexCoord1   : TEXCOORD1;  // tex coords
	float3 R   : TEXCOORD2;  // tex coords
};


VS_OUTPUT vertexMain( in float4 vPosition : POSITION,
                      in float3 vNormal   : NORMAL,
                      float2 texCoord0    : TEXCOORD0,
		      float2 texCoord1    : TEXCOORD1 )
{
	VS_OUTPUT Output;

	// transform position to clip space 
	Output.Position = mul(vPosition, mWorldViewProj);
	
	// transform normal 
	float3 normal = vNormal;
	normal.z = -normal.z;
	normal = mul(normal, mInvWorld);
	normal.z = -normal.z;
	
	// renormalize normal 
	normal = normalize(normal);
	
	// position in world coodinates
	float3 worldpos = mul(mTransWorld, vPosition);
	
	// calculate light vector, vtxpos - lightpos
	float3 lightVector = float3(0.0, 0.0, 0.0) - mLightPos;
	
	// normalize light vector 
	lightVector = normalize(lightVector);
	
	// calculate light color 
	float3 tmp = dot(-lightVector, normal);
	tmp = lit(tmp.x, tmp.y, 1.0);
	
	tmp = mLightColor * tmp.y;
	Output.Diffuse = float4(tmp.x, tmp.y, tmp.z, 0);
	Output.TexCoord0 = texCoord0;
	Output.TexCoord1 = texCoord1;
	
        float3 I = worldpos - eyePosition;
//        float3 tmp3;
//        tmp3.x = dot(normal, I);
//        tmp3 = normal * tmp3.x * 2.0;

//        Output.R.xyz = I - tmp3;
        Output.R = normalize(I - 2.0 * normal * dot(normal, I));
        Output.R.xz = normalize(Output.R.xz);
        Output.R.y = (Output.R.y * -0.5) + 0.5;
	
	
	return Output;
}



// Pixel shader output structure
struct PS_OUTPUT
{
    float4 RGBColor : COLOR0;  // Pixel color    
};


sampler2D tex0 : register(s0);
sampler2D tex1 : register(s1);
sampler2D tex2 : register(s2);
	
PS_OUTPUT pixelMain( float2 TexCoord0 : TEXCOORD0,
                     float2 TexCoord1 : TEXCOORD1,
                     float3 R : TEXCOORD2,
                     float4 Position  : POSITION,
                     float4 Diffuse   : COLOR0 ) 
{ 
	PS_OUTPUT Output;

	float4 col = tex2D( tex0, TexCoord0 );  // sample color map
	float dirt = 1.0 - tex2D( tex1, TexCoord0 ).x;  // sample color map

        float ex = ((R.x + R.z) + 1.415) / 2.83;
        if (R.x > R.z) ex = 1.0 - ex;
        float3 colenv = tex2D (tex2, float2(ex, R.y)).xyz * 0.8;
	
	Output.RGBColor = (Diffuse + float4(0.2, 0.2, 0.2, 1.0)) * col;
	Output.RGBColor *= 2.0;

	Output.RGBColor.xyz = lerp(Output.RGBColor.xyz, colenv, 0.3);
	Output.RGBColor.xyz = lerp(Output.RGBColor.xyz, float3(0.18,0.1,0.04), dirt);
	Output.RGBColor.w = 1.0;
	
	return Output;
}

